// Copyright 2018 The Amber Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef SRC_PARSER_H_
#define SRC_PARSER_H_

#include <memory>
#include <string>
#include <utility>

#include "amber/result.h"
#include "src/script.h"

namespace amber {

/// Base class for an input parser.
class Parser {
 public:
  virtual ~Parser();

  /// Run the parser against |data|. The result is success if the parse
  /// completes correctly.
  virtual Result Parse(const std::string& data) = 0;

  /// Retrieve the script which is generated by the parser.
  std::unique_ptr<Script> GetScript() { return std::move(script_); }

 protected:
  explicit Parser(Delegate* delegate);

  std::unique_ptr<Script> script_;
  Delegate* delegate_ = nullptr;
};

}  // namespace amber

#endif  // SRC_PARSER_H_
